﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Media.Imaging;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace HelloWorld
{
    /// <summary>
    ///     Interaction logic for PrintWindow.xaml
    /// </summary>
    public partial class PrintWindow : Window
    {
        public PrintWindow()
        {
            InitializeComponent();
        }

        public string ShpFilePath { get; set; }
        public string ShpFilePath2 { get; set; }




        private void PrintWindow_OnLoaded(object sender, RoutedEventArgs e)
        {
            try
            {
                if (string.IsNullOrEmpty(ShpFilePath))
                    return;

                SetupMapWithBlankPage();
                AddMapLayer();



            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message + " " + ex.StackTrace, "Exception");
            }
        }
        private void BtnjpgExport_OnClick(object sender, RoutedEventArgs e)
        {
            try
            {
                ExportToImageFormat(ImageFormat.Bmp);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message + " " + ex.StackTrace, "Exception");
            }
        }
        private void BtnTiffExport_OnClick(object sender, RoutedEventArgs e)
        {
            try
            {
                ExportToImageFormat(ImageFormat.Tiff);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message + " " + ex.StackTrace, "Exception");
            }


        }



        void SetupMapWithBlankPage()
        {


            // Setup the map unit, you want to always use feet or meters for printer layout maps
            Map1.MapUnit = GeographyUnit.Meter;

            // Here we create the default zoom levels for the sample.  We have pre-created a PrinterZoomLevelSet
            // That pre-defines commonly used zoom levels based on percentages of zoom
            Map1.ZoomLevelSet = new PrinterZoomLevelSet(Map1.MapUnit,
                PrinterHelper.GetPointsPerGeographyUnit(Map1.MapUnit));

            // Here we set the background color to gray so there is contrast with the while page
            Map1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.StandardColors.LightGray);

            // Create the PrinterInteractiveOverlay to contain all of the PrinterLayers.
            // The interactive overlay allows the layers to be interacted with
            var printerOverlay = new PrinterInteractiveOverlay();

            

            Map1.InteractiveOverlays.Add("PrintPreviewOverlay", printerOverlay);

            Map1.InteractiveOverlays.MoveToBottom("PrintPreviewOverlay");

            // Create the PagePrinterLayer which shows the page boundary and is the area the user will
            // arrange all of the other layer on top of.
            var pagePrinterLayer = new PagePrinterLayer(PrinterPageSize.AnsiA, PrinterOrientation.Portrait);
            pagePrinterLayer.Open();
            printerOverlay.PrinterLayers.Add("PageLayer", pagePrinterLayer);

            // Get the pages extent, slightly scale it up, and then set that as the default current extent
            Map1.CurrentExtent = AreaBaseShape.ScaleUp(pagePrinterLayer.GetPosition(), 10).GetBoundingBox();

            // Set the minimum sscale of the map to the last zoom level
            Map1.MinimumScale = Map1.ZoomLevelSet.ZoomLevel20.Scale;

        }
        void AddMapLayer()
        {
            var mapPrinterLayer = new MapPrinterLayer
            {
                MapUnit = GeographyUnit.Meter,
                BackgroundMask = new AreaStyle(new GeoPen(GeoColor.StandardColors.Black, 1))
            };

            mapPrinterLayer.Open();

            RectangleShape pageBoundingbox = GetPageBoundingBox(PrintingUnit.Inch);
            mapPrinterLayer.SetPosition(8, 7, pageBoundingbox.GetCenterPoint().X, pageBoundingbox.GetCenterPoint().Y + 1, PrintingUnit.Inch);


            var rectangleShape1 = AddShpFileToPrintLayer(mapPrinterLayer, ShpFilePath);




            var lst = new List<RectangleShape>();
            lst.AddMany(rectangleShape1);


            mapPrinterLayer.MapExtent = ExtentHelper.GetBoundingBoxOfItems(lst);



            var printerInteractiveOverlay = (PrinterInteractiveOverlay)Map1.InteractiveOverlays["PrintPreviewOverlay"];
            printerInteractiveOverlay.PrinterLayers.Add("MapLayer", mapPrinterLayer);



        }
        static RectangleShape AddShpFileToPrintLayer(MapPrinterLayer mapPrinterLayer, string shpFilePath)
        {
            var layer = new ShapeFileFeatureLayer(shpFilePath);
            layer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.County2;
            layer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.City1;
            layer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;


            mapPrinterLayer.Layers.Add(layer);
            layer.Open();
            var rectangleShape1 = layer.GetBoundingBox();
            layer.Close();
            return rectangleShape1;
        }
        RectangleShape GetPageBoundingBox(PrintingUnit unit)
        {
            var printerInteractiveOverlay = (PrinterInteractiveOverlay)Map1.InteractiveOverlays["PrintPreviewOverlay"];
            var pagePrinterLayer = (PagePrinterLayer)printerInteractiveOverlay.PrinterLayers["PageLayer"];
            return pagePrinterLayer.GetPosition(unit);
        }
        void ExportToImageFormat(ImageFormat format)
        {
            var printerInteractiveOverlay = (PrinterInteractiveOverlay)Map1.InteractiveOverlays["PrintPreviewOverlay"];
            var pagePrinterLayer = printerInteractiveOverlay.PrinterLayers["PageLayer"] as PagePrinterLayer;

            Bitmap bitmap = null;
            try
            {
                bitmap = new Bitmap((int)pagePrinterLayer.GetBoundingBox().Width, (int)pagePrinterLayer.GetBoundingBox().Height);

                var gdiPlusGeoCanvas = new GdiPlusGeoCanvas();
                gdiPlusGeoCanvas.BeginDrawing(bitmap, pagePrinterLayer.GetBoundingBox(), Map1.MapUnit);

                // Loop through all of the PrintingLayer in the PrinterInteractiveOverlay and print all of the
                // except for the PagePrinterLayer                
                var labelsInAllLayers = new Collection<SimpleCandidate>();
                foreach (var printerLayer in printerInteractiveOverlay.PrinterLayers)
                {
                    printerLayer.IsDrawing = true;
                    if (!(printerLayer is PagePrinterLayer))
                    {
                        printerLayer.Draw(gdiPlusGeoCanvas, labelsInAllLayers);
                    }
                    printerLayer.IsDrawing = false;
                }
                
                gdiPlusGeoCanvas.EndDrawing();

                var filename = "PrintingResults." + format;
                bitmap.Save(filename, format);
                Process.Start(filename);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message + " " + ex.StackTrace, "Exception");
            }
            finally
            {
                if (bitmap != null) { bitmap.Dispose(); }
            }
        }

      
    }





}